/**
 *
 * edgeLabel.java -- used in SEuS structure discovery system
 * Copyright (C) 2002 Shayan Ghazizadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/**
 * This class stores information about an edge label.
 * The information contains:
 *   - Label string
 *   - Index of the destination node in labels ArrayList!
 *   - Number of occurances
 */
public class edgeLabel implements Cloneable, java.io.Serializable {
    public String label;
    public int toIndex;
    public int fromIndex; //Only used in subgraph.countInst()!
    public int occ;
    public String annotation;

    public edgeLabel() {
	annotation = new String();
    }

    public int id;

    /**
     * Implements the classic equals method for this class.
     * Two labelNodes are equal, if the label strings are equal.
     */
    public boolean equals(Object o) {
	if (o instanceof edgeLabel) {
	    if (label.equals(((edgeLabel)o).label) && (toIndex == ((edgeLabel)o).toIndex))
		return true;
	}
	return false;
    }

    public void add(edgeLabel o) {
	if (equals(o)) {
	    occ += o.occ;
	}
	annotation = annotation + o.annotation;
    }

    /**
     * Implementation of Object.clone() method.
     */
    public Object clone() {
	edgeLabel copy = new edgeLabel();
	copy.occ = occ;
	copy.label = label;
	copy.toIndex = toIndex;
	copy.annotation = annotation;
	copy.fromIndex = fromIndex;
	copy.id = id;
	return copy;
    }
}
