#include <stdio.h>
#include <jni.h>
#include "subGraph.h"

#define MAXN 100
#include <nauty.h>
#include <naututil.h>

extern "C" JNIEXPORT jintArray JNICALL
Java_subGraph_getCanonC(JNIEnv *env, jobject obj, jintArray jAM, jintArray jC, jint jN, jint jL, jintArray jOrbit) {

  graph g[MAXN*MAXM], c[MAXN*MAXM];
  nvector lab[MAXN], ptn[MAXN], orbits[MAXN];
  static DEFAULTOPTIONS(options);
  statsblk(stats);
  setword workspace[50*MAXM];
  int m, n;
  jint *colors;
  jint *aMatrix;
  set *gv;
  
  // Get the data from VM!
  n = jN;
  m = (n + WORDSIZE - 1) / WORDSIZE;

  colors = env->GetIntArrayElements(jC, NULL);
  aMatrix = env->GetIntArrayElements(jAM, NULL);

  // Set-up the canonical, directed mode
  options.defaultptn = FALSE;
  options.getcanon = TRUE;
  options.digraph = TRUE;
  options.tc_level = 0;
  options.writeautoms = FALSE;
  options.writemarkers = FALSE;

  // define the graph
  for (int i=0; i<n; i++) {
    gv = GRAPHROW(g, i, m);
    EMPTYSET(gv, m);

    jint *p = aMatrix+i*n;
    for (int j=0; j<n; j++) {
      if (p[j] != 0)
	ADDELEMENT(gv, j);
    }
  }

  // define the partitions
  jint index = 0;
  for (jint i=0; i<jL; i++) {
    for (jint j=0; j<n; j++) {
      if (colors[j] == i) {
	lab[index] = j;
	ptn[index] = i+1;
	index++;
      }
    }
    ptn[index-1] = 0;
  }
  
  nauty(g, lab, ptn, NILSET, orbits, &options, &stats, workspace, 50*MAXM,
	m, n, c);

  //Cleanup
  env->ReleaseIntArrayElements(jC, colors, JNI_ABORT);
  env->ReleaseIntArrayElements(jAM, aMatrix, JNI_ABORT);

  jintArray result = env->NewIntArray(n);
  jint *lcopy = env->GetIntArrayElements(result, NULL);
  jint *lcopy2 = env->GetIntArrayElements(jOrbit, NULL);

  // copy lab into lcopy!
  for (int i=0; i<n; i++)
    lcopy[i] = lab[i];

  // Copy orbits into orbitCopy!
  for (int i=0; i<n; i++)
    lcopy2[i] = orbits[i];

  env->ReleaseIntArrayElements(result, lcopy, JNI_COMMIT);
  env->ReleaseIntArrayElements(jOrbit, lcopy2, JNI_COMMIT);
  return result;
}
