/**
 *
 * labelNode.java -- used in SEuS structure discovery system
 * Copyright (C) 2002 Shayan Ghazizadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/** 
 * Stores information about a label.
 * The information stored is
 *   - Label String
 *   - Number of occurances
 *   - List of labels that there is an edge from this label to them
 *
 * Note: A node can be a subgraph by itself if the system is used for
 * association rule generation!
 */
public class labelNode implements Cloneable, java.io.Serializable {
    public String label;
    public int occ;
    public subGraph sg;

    /**
     * Implements the classic equals method for this class.
     * Two labelNodes are equal, if the label strings are equal.
     */
    public boolean equals(Object o) {
	if (o instanceof labelNode) {
	    if (label.equals(((labelNode)o).label))
		return true;
	}
	return false;
    }

    /**
     * This method adds the number of occurances in object o to this
     * object in case of equality between the objects.
     */
    public void add(labelNode o) {
	if (equals(o)) {
	    occ += o.occ;
	}
    }

    /**
     * Implementation of Object.clone() method.
     */
    public Object clone() {
	labelNode copy = new labelNode();
	copy.occ = occ;
	copy.label = label;
	if (sg != null)
	    copy.sg = (subGraph)sg.clone();
	else
	    copy.sg = null;
	return copy;
    }

    public labelNode() {
	sg = null;
    }

    public labelNode(subGraph x) {
	sg = (subGraph)x.clone();
	if (sg.getF() != -1)
	    occ = sg.getF();
	else
	    occ = sg.getH();

	label = x.genLabel();
    }
}
