/**
 *
 * seus-exceptions.java -- exceptions thrown in different SEuS structure
 *                         discovery system modules
 * Copyright (C) 2002 Shayan Ghazizadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/**
 * This exception is thrown if you try to perform some
 * operation that needs the page to be in memory, but
 * your page is on disk.
 *
 * @author Shayan Ghazizadeh <shayan@cs.umd.edu>
 */
class PageNotInMemoryException extends Exception {
}

/**
 * This exception is thrown if you try to add a new
 * value to a page that is already full.
 *
 * @author Shayan Ghazizadeh <shayan@cs.umd.edu>
 */
class PageIsFullException extends Exception {
    public PageIsFullException() {}
    public PageIsFullException(String msg) {
	super(msg);
    }
}

/**
 * This exception is thrown if you try to perform an add
 * operation while the object is in get mode. When an
 * object goes into get mode, you can not add anymore
 * values to it!
 *
 * @author Shayan Ghazizadeh <shayan@cs.umd.edu>
 */
class AddWhileInGetModeException extends Exception {
}


class SleepWhileInAddModeException extends Exception {
}
