/**
 *
 * seusGUI.java -- GUI for SEuS structure discovery system
 * Copyright (C) 2002 Shayan Ghazizadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version as long
 * as the following "Sun" copyright notice and the licence apear
 * on all copies of the software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * @(#)Graph.java	1.9 99/08/04
 *
 * Copyright (c) 1997, 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class Node {
    double x;
    double y;

    double dx;
    double dy;

    boolean fixed;

    String lbl;
}


class Edge {
    int from;
    int to;

    double len;
    String lbl;
}


class GraphPanel extends JPanel
    implements ActionListener, MouseListener, MouseMotionListener {
    seusGUI graph;
    int nnodes;
    Node nodes[] = new Node[100];

    int nedges;
    Edge edges[] = new Edge[200];

    javax.swing.Timer timer;
    boolean stress;
    boolean random;

    GraphPanel(seusGUI graph) {
	this.graph = graph;

	timer = new javax.swing.Timer(100, this);
        timer.setInitialDelay(0);
        timer.setCoalesce(true);

	addMouseListener(this);
    }

    int findNode(String lbl) {
	for (int i = 0 ; i < nnodes ; i++) {
	    if (nodes[i].lbl.equals(lbl)) {
		return i;
	    }
	}
	return addNode(lbl);
    }
    int addNode(String lbl) {
	Node n = new Node();
	n.x = 10 + 380*Math.random();
	n.y = 10 + 380*Math.random();
	n.lbl = lbl;
	nodes[nnodes] = n;
	return nnodes++;
    }
    void addEdge(String from, String to, String lbl, int len) {
	Edge e = new Edge();
	e.from = findNode(from);
	e.to = findNode(to);
	e.lbl = lbl;
	e.len = len;
	edges[nedges++] = e;
    }

    public void run() {
	/*
        Thread me = Thread.currentThread();
	while (relaxer == me) {
	    relax();
	    if (random && (Math.random() < 0.03)) {
		Node n = nodes[(int)(Math.random() * nnodes)];
		if (!n.fixed) {
		    n.x += 100*Math.random() - 50;
		    n.y += 100*Math.random() - 50;
		}
	    }
	    
	    try {
		Thread.sleep(100);
	    } catch (InterruptedException e) {
		break;
	    }
	    
	}
	*/
    }

    public void actionPerformed(ActionEvent e) {
	relax();
	if (random && (Math.random() < 0.03)) {
	    Node n = nodes[(int)(Math.random() * nnodes)];
	    if (!n.fixed) {
		n.x += 100*Math.random() - 50;
		n.y += 100*Math.random() - 50;
	    }
	}
    }

    synchronized void relax() {
	for (int i = 0 ; i < nedges ; i++) {
	    Edge e = edges[i];
	    double vx = nodes[e.to].x - nodes[e.from].x;
	    double vy = nodes[e.to].y - nodes[e.from].y;
	    double len = Math.sqrt(vx * vx + vy * vy);
            len = (len == 0) ? .0001 : len;
	    double f = (edges[i].len - len) / (len * 3);
	    double dx = f * vx;
	    double dy = f * vy;

	    nodes[e.to].dx += dx;
	    nodes[e.to].dy += dy;
	    nodes[e.from].dx += -dx;
	    nodes[e.from].dy += -dy;
	}

	for (int i = 0 ; i < nnodes ; i++) {
	    Node n1 = nodes[i];
	    double dx = 0;
	    double dy = 0;

	    for (int j = 0 ; j < nnodes ; j++) {
		if (i == j) {
		    continue;
		}
		Node n2 = nodes[j];
		double vx = n1.x - n2.x;
		double vy = n1.y - n2.y;
		double len = vx * vx + vy * vy;
		if (len == 0) {
		    dx += Math.random();
		    dy += Math.random();
		} else if (len < 100*100) {
		    dx += vx / len;
		    dy += vy / len;
		}
	    }
	    double dlen = dx * dx + dy * dy;
	    if (dlen > 0) {
		dlen = Math.sqrt(dlen) / 2;
		n1.dx += dx / dlen;
		n1.dy += dy / dlen;
	    }
	}

	Dimension d = getSize();
	for (int i = 0 ; i < nnodes ; i++) {
	    Node n = nodes[i];
	    if (!n.fixed) {
		n.x += Math.max(-5, Math.min(5, n.dx));
		n.y += Math.max(-5, Math.min(5, n.dy));
            }
            if (n.x < 0) {
                n.x = 0;
            } else if (n.x > d.width) {
                n.x = d.width;
            }
            if (n.y < 0) {
                n.y = 0;
            } else if (n.y > d.height) {
                n.y = d.height;
            }
	    n.dx /= 2;
	    n.dy /= 2;
	}
	repaint();
    }

    Node pick;
    boolean pickfixed;
    Image offscreen;
    Dimension offscreensize;
    Graphics offgraphics;

    final Color fixedColor = Color.red;
    final Color selectColor = Color.pink;
    final Color edgeColor = Color.black;
    final Color nodeColor = new Color(250, 220, 100);
    final Color stressColor = Color.darkGray;
    final Color arcColor1 = Color.black;
    final Color arcColor2 = Color.pink;
    final Color arcColor3 = Color.red;

    public void paintNode(Graphics g, Node n, FontMetrics fm) {
	int x = (int)n.x;
	int y = (int)n.y;
	g.setColor((n == pick) ? selectColor : (n.fixed ? fixedColor : nodeColor));
	int w = fm.stringWidth(n.lbl) + 10;
	int h = fm.getHeight() + 4;
	g.fillRect(x - w/2, y - h / 2, w, h);
	g.setColor(Color.black);
	g.drawRect(x - w/2, y - h / 2, w-1, h-1);
	g.drawString(n.lbl, x - (w-10)/2, (y - (h-4)/2) + fm.getAscent());
    }

    public synchronized void paintComponent(Graphics g) {
	super.paintComponent(g); //paint background
	Dimension d = getSize();
	if ((offscreen == null) || (d.width != offscreensize.width) || (d.height != offscreensize.height)) {
	    offscreen = createImage(d.width, d.height);
	    offscreensize = d;
	    if (offgraphics != null) {
	        offgraphics.dispose();
	    }
	    offgraphics = offscreen.getGraphics();
	    offgraphics.setFont(getFont());
	}

	offgraphics.setColor(getBackground());
	offgraphics.fillRect(0, 0, d.width, d.height);
	for (int i = 0 ; i < nedges ; i++) {
	    Edge e = edges[i];
	    int x1 = (int)nodes[e.from].x;
	    int y1 = (int)nodes[e.from].y;
	    int x2 = (int)nodes[e.to].x;
	    int y2 = (int)nodes[e.to].y;
	    int len = (int)Math.abs(Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)) - e.len) + 40;
	    offgraphics.setColor((len < 10) ? arcColor1 : (len < 20 ? arcColor2 : arcColor3)) ;
	    offgraphics.drawLine(x1, y1, x2, y2);
	    if (stress) {
		String lbl = String.valueOf(len);
		offgraphics.setColor(stressColor);
		offgraphics.drawString(lbl, x1 + (x2-x1)/2, y1 + (y2-y1)/2);
		offgraphics.setColor(edgeColor);
	    }
	    else {
		offgraphics.setColor(stressColor);
		offgraphics.drawString(e.lbl, x1 + (x2-x1)/2, y1 + (y2-y1)/2);
		offgraphics.setColor(edgeColor);
	    }
	}

	FontMetrics fm = offgraphics.getFontMetrics();
	for (int i = 0 ; i < nnodes ; i++) {
	    paintNode(offgraphics, nodes[i], fm);
	}
	g.drawImage(offscreen, 0, 0, null);
    }

    //1.1 event handling
    public void mouseClicked(MouseEvent e) {
	graph.setFocus(this);
    }

    public void mousePressed(MouseEvent e) {
        addMouseMotionListener(this);
	double bestdist = Double.MAX_VALUE;
	int x = e.getX();
	int y = e.getY();
	for (int i = 0 ; i < nnodes ; i++) {
	    Node n = nodes[i];
	    double dist = (n.x - x) * (n.x - x) + (n.y - y) * (n.y - y);
	    if (dist < bestdist) {
		pick = n;
		bestdist = dist;
	    }
	}
	pickfixed = pick.fixed;
	pick.fixed = true;
	pick.x = x;
	pick.y = y;
	repaint();
	e.consume();
    }

    public void mouseReleased(MouseEvent e) {
        removeMouseMotionListener(this);
        if (pick != null) {
            pick.x = e.getX();
            pick.y = e.getY();
            pick.fixed = pickfixed;
            pick = null;
        }
	repaint();
	e.consume();
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
	pick.x = e.getX();
	pick.y = e.getY();
	repaint();
	e.consume();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void start() {
	if (!timer.isRunning()) {
	    timer.start();
	}
	//relaxer = new Thread(this);
	//relaxer.start();
    }

    public void stop() {
	if (timer.isRunning()) {
            timer.stop();
        }
	//relaxer = null;
    }

    public void toggle() {
	if (timer.isRunning()) {
            timer.stop();
        }
	else
	    timer.start();
    }
}


public class seusGUI extends JFrame implements ActionListener, ItemListener {

    static final int width = 800;
    static final int height = 700;
    static final int controlH = 100;
    static final int margin = 5;
    
    seus sdm;
    int sLevel, bw;

    JPanel mainPanel;
    JScrollPane view;
    GraphPanel [] panels;
    JPanel controlPanel;

    // Buttons
    JButton scramble = new JButton("Scramble All Graphs");
    JButton shake = new JButton("Toggle All Animations");
    JSlider comp = new JSlider(0, 100, 90);
    JButton search = new JButton("Search");
    JButton count = new JButton("Count");
    JLabel label = new JLabel("Current SCM: 0.9");
    JButton quit = new JButton("Quit");

    // Borders
    Border etched, loweredlevel;
    TitledBorder titles[];

    int currFocus;

    boolean inCount;
    ArrayList best;

    /**
     * constructor.
     */
    public seusGUI(seus lsdm, int sl, int lbw) {
	sdm = lsdm;
	sLevel = sl;
	bw = lbw;
	inCount = false;
	best = sdm.tempBest;

	// Defining general borders
	etched = BorderFactory.createEtchedBorder();
	loweredlevel = BorderFactory.createLoweredBevelBorder();

	// gracefully exit on closing of window
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
	}); 

	Container cp = getContentPane();
	setBounds(0, 0, width+4*margin, height+margin);
	cp.setLayout(null);

	
	
	controlPanel = new JPanel();
	controlPanel.setBounds(0, 0, 2*width/3+30, controlH);
	cp.add(controlPanel);

	
	mainPanel = new JPanel();
	mainPanel.setBounds(0, 110, width, height-controlH);
	cp.add(mainPanel);
	mainPanel.setLayout(null);

	
	panels = new GraphPanel[4];
	titles = new TitledBorder[4];
	for (int i=0; i<4; i++) {
	    if (i >= best.size())
		continue;
	    panels[i] = new GraphPanel(this);
	    if (i == 0) {
		titles[i] = BorderFactory.createTitledBorder(loweredlevel, "Sub "+(i+1)+
							     "    E = "+ ((subGraph)best.get(i)).getH()+
							     "    C = "+ ((subGraph)best.get(i)).complexity());
		currFocus = 0;
	    }
	    else
		titles[i] = BorderFactory.createTitledBorder(etched, "Sub "+(i+1)+
							     "    E = "+ ((subGraph)best.get(i)).getH()+
							     "    C = "+ ((subGraph)best.get(i)).complexity());
	    panels[i].setBorder(titles[i]);
	    switch (i) {
	    case 0: panels[i].setBounds(0, 0, width/2 - margin, (height-controlH)/2); break;
	    case 1: panels[i].setBounds(width/2 + margin, 0, width/2, (height-controlH)/2); break;
	    case 2: panels[i].setBounds(0, (height-controlH)/2, width/2 - margin, (height-controlH)/2); break;
	    case 3: panels[i].setBounds(width/2 + margin, (height-controlH)/2, width/2, (height-controlH)/2); break;
	    }
	    mainPanel.add(panels[i]);
	}
	
	GridBagLayout gridbagCon = new GridBagLayout();
       	controlPanel.setLayout(gridbagCon);

	Hashtable compLabel = new Hashtable();
	for (int i=0; i<=100; i+=10) {
	    compLabel.put(new Integer(i), new JLabel(String.valueOf((double)i/100)));
	}
	comp.setLabelTable(compLabel);
	comp.setPaintLabels(true);
	
	

	
	GridBagConstraints c = new GridBagConstraints();
	c.ipadx = c.ipady = 5;
	c.fill = GridBagConstraints.HORIZONTAL;

	// Search Button!
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	gridbagCon.setConstraints(search, c);
	controlPanel.add(search); 
	search.addActionListener(this);
	
	// Count Button!
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	gridbagCon.setConstraints(count, c);
	controlPanel.add(count); 
	count.addActionListener(this);


	// Shake Button!
	c.gridx = 2;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	gridbagCon.setConstraints(shake, c);
	controlPanel.add(shake); 
	shake.addActionListener(this);

	// Scramble Button!
	c.gridx = 3;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	gridbagCon.setConstraints(scramble, c);
	controlPanel.add(scramble); 
	scramble.addActionListener(this);

	// Slider!
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridheight = 1;
	c.fill = GridBagConstraints.HORIZONTAL;
	gridbagCon.setConstraints(comp, c);
	controlPanel.add(comp);

	// Label!
	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 3;
	c.gridheight = 1;
	//	c.fill = GridBagConstraints.NONE;
	gridbagCon.setConstraints(label, c);
	controlPanel.add(label); 
	
	// Quit Button!
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	gridbagCon.setConstraints(quit, c);
	controlPanel.add(quit); 
	quit.addActionListener(this);

	int upto = best.size();
	if (upto > 4)
	    upto = 4;
	for (int i=0; i<upto; i++) {
	    subGraph sub = (subGraph)best.get(i);
	    ListIterator it = sub.edges.listIterator();
	    boolean added = false;
	    while (it.hasNext()) {
		ArrayList lFanOut = (ArrayList)it.next();
		if (lFanOut != null) {
		    added = true;
		    Iterator it2 = lFanOut.iterator();
		    while(it2.hasNext()) {
			edgeLabel e = (edgeLabel)it2.next();
			panels[i].addEdge(String.valueOf(it.previousIndex())+":"+((labelNode)sub.labels.get(it.nextIndex()-1)).label, 
					  String.valueOf(e.toIndex)+":"+((labelNode)sub.labels.get(e.toIndex)).label, e.label, 50);
		    }
		}
	    }
	    if (!added) {
		panels[i].addEdge("0:"+((labelNode)sub.labels.get(0)).label, "0:"+((labelNode)sub.labels.get(0)).label, "", 50);
	    }
	}
	
	setVisible(true);
    }

    public void setFocus(GraphPanel gp) {
	int set2 = 0;
	for (int i=0; i<4; i++) {
	    if (i >= best.size())
		continue;
	    if (gp == panels[i]) {
		set2 = i;
		break;
	    }
	}
	if (set2 != currFocus) {
	    if (inCount) {
		titles[set2] = BorderFactory.createTitledBorder(loweredlevel, "Sub "+(set2+1)+
								"    E = "+ ((subGraph)best.get(set2)).getH()+
								"    A = "+ ((subGraph)best.get(set2)).getF()+
								"    C = "+ ((subGraph)best.get(set2)).complexity());
		panels[set2].setBorder(titles[set2]);
		titles[currFocus] = BorderFactory.createTitledBorder(etched, "Sub "+(currFocus+1)+
								     "    E = "+ ((subGraph)best.get(currFocus)).getH()+
								     "    A = "+ ((subGraph)best.get(currFocus)).getF()+
								     "    C = "+ ((subGraph)best.get(currFocus)).complexity());
	    }
	    else {
		titles[set2] = BorderFactory.createTitledBorder(loweredlevel, "Sub "+(set2+1)+
								"    E = "+ ((subGraph)best.get(set2)).getH()+
								"    C = "+ ((subGraph)best.get(set2)).complexity());
		panels[set2].setBorder(titles[set2]);
		titles[currFocus] = BorderFactory.createTitledBorder(etched, "Sub "+(currFocus+1)+
								     "    E = "+ ((subGraph)best.get(currFocus)).getH()+
								     "    C = "+ ((subGraph)best.get(currFocus)).complexity());
	    }
	    panels[currFocus].setBorder(titles[currFocus]);
	    currFocus = set2;
	}
    }


    public void destroy() {
	for (int i=0; i<4; i++) {
	    if (i >= best.size())
		continue;
	    mainPanel.remove(panels[i]);
	}
        //remove(controlPanel);
    }

    public void start() {
	for (int i=0; i<4; i++) {
	    if (i >= best.size())
		continue;
	    panels[i].start();
	}
    }

    public void stop() {
	for (int i=0; i<4; i++) {
	    if (i >= best.size())
		continue;
	    panels[i].stop();
	}
    }

    public void actionPerformed(ActionEvent e) {
	Object src = e.getSource();

	if (src == scramble) {
	    for (int j=0; j<4; j++) {
		panels[j].start();
		Dimension d = getSize();
		for (int i = 0 ; i < panels[j].nnodes ; i++) {
		    Node n = panels[j].nodes[i];
		    if (!n.fixed) {
			n.x = 10 + (d.width-20)*Math.random();
			n.y = 10 + (d.height-20)*Math.random();
		    }
		}
	    }
	    return;
	}

	if (src == shake) {
	    for (int i=0; i<4; i++)
		panels[i].toggle();
	    /*
	    Dimension d = getSize();
	    for (int i = 0 ; i < panels[currFocus].nnodes ; i++) {
		Node n = panels[currFocus].nodes[i];
		if (!n.fixed) {
		    n.x += 80*Math.random() - 40;
		    n.y += 80*Math.random() - 40;
		}
	    }
	    */
	}

	if (src == quit) {
	    seus.done(sdm);
	    System.exit(0);
	}

	if (src == search) {
	    stop();
	    destroy();

	    seus temp = seus.doSearch(sdm.dg, sLevel, bw, (double)comp.getValue()/100);
	    sdm = temp;
	    best = sdm.tempBest;
	    inCount = false;

	    label.setText("Current SCM: " + ((double)comp.getValue()/100));

	    for (int i=0; i<4; i++) {
		if (i >= best.size())
		    continue;
		panels[i] = new GraphPanel(this);
		if (i == 0) {
		    titles[i] = BorderFactory.createTitledBorder(loweredlevel, "Sub "+(i+1)+"    E = "+
								 ((subGraph)best.get(i)).getH()+"    C = "+
								 ((subGraph)best.get(i)).complexity());
		    currFocus = 0;
		}
		else
		    titles[i] = BorderFactory.createTitledBorder(etched, "Sub "+(i+1)+"    E = "+
								 ((subGraph)best.get(i)).getH()+"    C = "+
								 ((subGraph)best.get(i)).complexity());
		panels[i].setBorder(titles[i]);
		switch (i) {
		case 0: panels[i].setBounds(0, 0, width/2 - margin, (height-controlH)/2); break;
		case 1: panels[i].setBounds(width/2 + margin, 0, width/2, (height-controlH)/2); break;
		case 2: panels[i].setBounds(0, (height-controlH)/2, width/2 - margin, (height-controlH)/2); break;
		case 3: panels[i].setBounds(width/2 + margin, (height-controlH)/2, width/2, (height-controlH)/2); break;
		}
		mainPanel.add(panels[i]);
	    }
	    
	    
	    int upto = best.size();
	    if (upto > 4)
		upto = 4;
	    for (int i=0; i<upto; i++) {
		subGraph sub = (subGraph)best.get(i);
		ListIterator it = sub.edges.listIterator();
		boolean added = false;
		while (it.hasNext()) {
		    ArrayList lFanOut = (ArrayList)it.next();
		    if (lFanOut != null) {
			added = true;
			Iterator it2 = lFanOut.iterator();
			while(it2.hasNext()) {
			    edgeLabel ed = (edgeLabel)it2.next();
			    panels[i].addEdge(String.valueOf(it.previousIndex())+":"+((labelNode)sub.labels.get(it.nextIndex()-1)).label, 
					  String.valueOf(ed.toIndex)+":"+((labelNode)sub.labels.get(ed.toIndex)).label, ed.label, 50);
			}
		    }
		}
		if (!added) {
		    panels[i].addEdge("0:"+((labelNode)sub.labels.get(0)).label, "0:"+((labelNode)sub.labels.get(0)).label, "", 50);
		}
	    }
	    
	    start();
	}

	if (src == count) {
	    stop();
	    destroy();

	    best = seus.doCount(sdm);
	    inCount = true;

	    label.setText("Current SCM: " + ((double)comp.getValue()/100));

	    for (int i=0; i<4; i++) {
		if (i >= best.size())
		    continue;
		panels[i] = new GraphPanel(this);
		if (i == 0) {
		    titles[i] = BorderFactory.createTitledBorder(loweredlevel, "Sub "+(i+1)+
								 "    E = "+ ((subGraph)best.get(i)).getH()+
								 "    A = "+ ((subGraph)best.get(i)).getF()+
								 "    C = "+ ((subGraph)best.get(i)).complexity());
		    currFocus = 0;
		}
		else
		    titles[i] = BorderFactory.createTitledBorder(etched, "Sub "+(i+1)+
								 "    E = "+ ((subGraph)best.get(i)).getH()+
								 "    A = "+ ((subGraph)best.get(i)).getF()+
								 "    C = "+ ((subGraph)best.get(i)).complexity());
		panels[i].setBorder(titles[i]);
		switch (i) {
		case 0: panels[i].setBounds(0, 0, width/2 - margin, (height-controlH)/2); break;
		case 1: panels[i].setBounds(width/2 + margin, 0, width/2, (height-controlH)/2); break;
		case 2: panels[i].setBounds(0, (height-controlH)/2, width/2 - margin, (height-controlH)/2); break;
		case 3: panels[i].setBounds(width/2 + margin, (height-controlH)/2, width/2, (height-controlH)/2); break;
		}
		mainPanel.add(panels[i]);
	    }
	    
	    
	    int upto = best.size();
	    if (upto > 4)
		upto = 4;
	    for (int i=0; i<upto; i++) {
		subGraph sub = (subGraph)best.get(i);
		ListIterator it = sub.edges.listIterator();
		boolean added = false;
		while (it.hasNext()) {
		    ArrayList lFanOut = (ArrayList)it.next();
		    if (lFanOut != null) {
			added = true;
			Iterator it2 = lFanOut.iterator();
			while(it2.hasNext()) {
			    edgeLabel ed = (edgeLabel)it2.next();
			    panels[i].addEdge(String.valueOf(it.previousIndex())+":"+((labelNode)sub.labels.get(it.nextIndex()-1)).label, 
					  String.valueOf(ed.toIndex)+":"+((labelNode)sub.labels.get(ed.toIndex)).label, ed.label, 50);
			}
		    }
		}
		if (!added) {
		    panels[i].addEdge("0:"+((labelNode)sub.labels.get(0)).label, "0:"+((labelNode)sub.labels.get(0)).label, "", 50);
		}
	    }
	    
	    start();
	}
    }

    public void itemStateChanged(ItemEvent e) {
	Object src = e.getSource();
	boolean on = e.getStateChange() == ItemEvent.SELECTED;
    }

    public static void usage() {
	System.err.println("Usage: seusGUI [-xml] [-scm i] [-l0|-l1] input-file");
	System.err.println("\t -xml     : Treat input file as a XML file.");
	System.err.println("\t -scm i   : The structure complexity measure. default is 1.0 [approximate mode]");
	System.err.println("\t -l0|-l1  : Select summary level. L0 is default.");
    }

    public static void main(String[] argv) {
	String filename = null;
	boolean xml = false;
	double lscm = 1;
	int lbw=4;
	int sLevel = 0;

	// Parse arguments
	for(int i=0; i<argv.length; i++) {
	    if (argv[i].equals("-xml")) {
		// Input file is an XML file, it must be converted first!
		xml = true;
	    }
	    else if (argv[i].equals("-scm")) {
		lscm = Double.valueOf(argv[i+1]).doubleValue();
		i++;
	    }
	    else if (argv[i].equals("-bw")) {
		lbw = Integer.valueOf(argv[i+1]).intValue();
		i++;
	    }
	    else if (argv[i].equals("-l0")) {
		sLevel = 0;
	    }
	    else if (argv[i].equals("-l1")) {
		sLevel = 1;
	    }
	    else
		filename = argv[i];
	}

	if (filename == null) {
	    usage();
	}
	else {
	    dataGuide dg = seus.init(filename, sLevel, xml);
	    if (dg != null) {
		seus sdm = seus.doSearch(dg, sLevel, lbw, lscm);
		seusGUI my_frame = new seusGUI(sdm, sLevel, lbw);
		my_frame.start();
	    }
	}
    }
}


